@@ -22,14 +22,36 @@ use crate::font_introspector::text::Script;
 use crate::font_introspector::{CacheKey, FontRef, Synthesis};
 use crate::layout::FragmentStyle;
 use crate::SugarloafErrors;
+use dashmap::DashMap;
 use parking_lot::RwLock;
 use rustc_hash::FxHashMap;
 use std::ops::Range;
 use std::path::PathBuf;
-use std::sync::Arc;
+use std::sync::{Arc, OnceLock};
 
 pub use crate::font_introspector::{Style, Weight};
 
+// Type alias for the font data cache to improve readability
+type FontDataCache = Arc<DashMap<PathBuf, SharedData>>;
+
+// Global font data cache to avoid reloading fonts from disk
+// This cache stores font file data indexed by path, so fonts are only loaded once
+// and shared across all font library instances. This significantly improves
+// performance when the same font is referenced multiple times.
+static FONT_DATA_CACHE: OnceLock<FontDataCache> = OnceLock::new();
+
+fn get_font_data_cache() -> &'static FontDataCache {
+    FONT_DATA_CACHE.get_or_init(|| Arc::new(DashMap::default()))
+}
+
+/// Clears the global font data cache, forcing fonts to be reloaded from disk
+/// on next access. This should be called when font configuration changes.
+pub fn clear_font_data_cache() {
+    if let Some(cache) = FONT_DATA_CACHE.get() {
+        cache.clear();
+    }
+}
+
 pub fn lookup_for_font_match(
     cluster: &mut CharCluster,
     synth: &mut Synthesis,
@@ -224,10 +246,9 @@ impl FontLibraryData {
             if let Some(data) = &font.data {
                 return Some((data.clone(), font.offset, font.key));
             } else if let Some(path) = &font.path {
-                // Load font data directly from file when needed
+                // Load font data from cache or disk
                 if let Some(raw_data) = load_from_font_source(path) {
-                    let shared_data = SharedData::new(raw_data);
-                    return Some((shared_data, font.offset, font.key));
+                    return Some((raw_data, font.offset, font.key));
                 }
             }
         }
@@ -314,8 +335,6 @@ impl FontLibraryData {
         }
 
         let mut db = loader::Database::new();
-        db.load_system_fonts();
-
         spec.additional_dirs
             .unwrap_or_default()
             .into_iter()
@@ -506,16 +525,16 @@ impl FontLibraryData {
 }
 
 /// Atomically reference counted, heap allocated or memory mapped buffer.
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct SharedData {
-    inner: Arc<dyn AsRef<[u8]> + Send + Sync>,
+    inner: Arc<[u8]>,
 }
 
 impl SharedData {
     /// Creates shared data from the specified bytes.
     pub fn new(data: Vec<u8>) -> Self {
         Self {
-            inner: Arc::new(data),
+            inner: Arc::from(data),
         }
     }
 }
@@ -631,13 +650,14 @@ impl FontData {
 
     #[inline]
     pub fn from_data(
-        data: Vec<u8>,
+        data: SharedData,
         path: PathBuf,
         evictable: bool,
         is_emoji: bool,
         font_spec: &SugarloafFont,
     ) -> Result<Self, Box<dyn std::error::Error>> {
-        let font = FontRef::from_index(&data, 0).unwrap();
+        let font = FontRef::from_index(&data, 0)
+            .ok_or_else(|| format!("Failed to load font from path: {:?}", path))?;
         let (offset, key) = (font.offset, font.key);
 
         // Return our struct with the original file data and copies of the
@@ -654,11 +674,7 @@ impl FontData {
         let stretch = attributes.stretch();
         let synth = attributes.synthesize(attributes);
 
-        let data = if evictable {
-            None
-        } else {
-            Some(SharedData::new(data))
-        };
+        let data = (!evictable).then_some(data);
 
         Ok(Self {
             data,
@@ -727,8 +743,6 @@ fn find_font(
     evictable: bool,
     is_emoji: bool,
 ) -> FindResult {
-    use std::io::Read;
-
     if !font_spec.is_default_family() {
         let family = font_spec.family.to_string();
         let mut query = crate::font::loader::Query {
@@ -775,14 +789,14 @@ fn find_font(
 
         match db.query(&query) {
             Some(id) => {
-                if let Some((crate::font::loader::Source::File(ref path), _index)) =
-                    db.face_source(id)
-                {
-                    if let Ok(mut file) = std::fs::File::open(path) {
-                        let mut font_data = vec![];
-                        if file.read_to_end(&mut font_data).is_ok() {
+                match db.face_source(id) {
+                    Some((crate::font::loader::Source::File(ref path), _index)) => {
+                        // File source - load from path
+                        if let Some(font_data_arc) =
+                            load_from_font_source(&path.to_path_buf())
+                        {
                             match FontData::from_data(
-                                font_data,
+                                font_data_arc,
                                 path.to_path_buf(),
                                 evictable,
                                 is_emoji,
@@ -805,6 +819,35 @@ fn find_font(
                             }
                         }
                     }
+                    Some((crate::font::loader::Source::Binary(font_data), _index)) => {
+                        // Binary source - use data directly
+                        tracing::debug!(
+                            "Using binary font data, {} bytes",
+                            font_data.len()
+                        );
+                        // Convert Arc<Vec<u8>> to SharedData
+                        match FontData::from_data(
+                            font_data,
+                            std::path::PathBuf::from(&family),
+                            evictable,
+                            is_emoji,
+                            &font_spec,
+                        ) {
+                            Ok(d) => {
+                                tracing::info!("Font '{}' loaded from memory", family);
+                                return FindResult::Found(d);
+                            }
+                            Err(err_message) => {
+                                tracing::info!(
+                                    "Failed to load font '{query:?}' from memory, {err_message}"
+                                );
+                                return FindResult::NotFound(font_spec);
+                            }
+                        }
+                    }
+                    None => {
+                        tracing::warn!("face_source returned None for font ID");
+                    }
                 }
             }
             None => {
@@ -878,13 +921,26 @@ fn find_font_path(
 }
 
 #[cfg(not(target_arch = "wasm32"))]
-fn load_from_font_source(path: &PathBuf) -> Option<Vec<u8>> {
+fn load_from_font_source(path: &PathBuf) -> Option<SharedData> {
     use std::io::Read;
 
+    let cache = get_font_data_cache();
+
+    // Check if already cached - DashMap handles concurrent access efficiently
+    if let Some(cached_data) = cache.get(path) {
+        return Some(cached_data.clone());
+    }
+
+    // Load from disk if not cached
     if let Ok(mut file) = std::fs::File::open(path) {
         let mut font_data = vec![];
         if file.read_to_end(&mut font_data).is_ok() {
-            return Some(font_data);
+            let shared_data = SharedData::new(font_data);
+            // Use entry API to handle concurrent inserts properly
+            let entry = cache
+                .entry(path.clone())
+                .or_insert_with(|| shared_data.clone());
+            return Some(entry.clone());
         }
     }
 
