@@ -3,20 +3,23 @@
 // This source code is licensed under the MIT license found in the
 // LICENSE file in the root directory of this source tree.
 
-use super::compositors::SugarCompositors;
 use crate::font::FontLibrary;
 use crate::layout::RootStyle;
-use crate::sugarloaf::{text, QuadBrush, RectBrush, RichTextBrush, RichTextLayout};
-use crate::SugarDimensions;
-use crate::{Content, Graphics, Object, RichText};
+use crate::sugarloaf::QuadBrush;
+use crate::sugarloaf::{RichTextBrush, RichTextLayout};
+use crate::Graphics;
+use crate::{Content, Object, Quad, RichText, SugarDimensions};
 use std::collections::HashSet;
 
 pub struct SugarState {
     objects: Vec<Object>,
     pub rich_texts: Vec<RichText>,
     rich_text_repaint: HashSet<usize>,
+    rich_text_to_be_removed: Vec<usize>,
     pub style: RootStyle,
-    pub compositors: SugarCompositors,
+    pub content: Content,
+    pub quads: Vec<Quad>,
+    pub visual_bell_overlay: Option<Quad>,
 }
 
 impl SugarState {
@@ -25,21 +28,43 @@ impl SugarState {
         font_library: &FontLibrary,
         font_features: &Option<Vec<String>>,
     ) -> SugarState {
-        let mut state = SugarState {
-            compositors: SugarCompositors::new(font_library),
+        let mut content = Content::new(font_library);
+        let found_font_features = SugarState::found_font_features(font_features);
+        content.set_font_features(found_font_features);
+
+        SugarState {
+            content: Content::new(font_library),
+            quads: vec![],
             style,
             objects: vec![],
             rich_texts: vec![],
+            rich_text_to_be_removed: vec![],
             rich_text_repaint: HashSet::default(),
-        };
+            visual_bell_overlay: None,
+        }
+    }
 
-        state.compositors.advanced.set_font_features(font_features);
-        state
+    pub fn found_font_features(
+        font_features: &Option<Vec<String>>,
+    ) -> Vec<crate::font_introspector::Setting<u16>> {
+        let mut found_font_features = vec![];
+        if let Some(features) = font_features {
+            for feature in features {
+                let setting: crate::font_introspector::Setting<u16> =
+                    (feature.as_str(), 1).into();
+                found_font_features.push(setting);
+            }
+        }
+
+        found_font_features
     }
 
+    #[inline]
+    pub fn new_layer(&mut self) {}
+
     #[inline]
     pub fn get_state_layout(&self, id: &usize) -> RichTextLayout {
-        if let Some(builder_state) = self.compositors.advanced.content.get_state(id) {
+        if let Some(builder_state) = self.content.get_state(id) {
             return builder_state.layout;
         }
 
@@ -53,7 +78,7 @@ impl SugarState {
         advance_brush: &mut RichTextBrush,
     ) {
         self.style.scale_factor = scale;
-        for (id, state) in &mut self.compositors.advanced.content.states {
+        for (id, state) in &mut self.content.states {
             state.rescale(scale);
             state.layout.dimensions.height = 0.0;
             state.layout.dimensions.width = 0.0;
@@ -71,12 +96,7 @@ impl SugarState {
         font_size: f32,
         advance_brush: &mut RichTextBrush,
     ) {
-        if let Some(rte) = self
-            .compositors
-            .advanced
-            .content
-            .get_state_mut(rich_text_id)
-        {
+        if let Some(rte) = self.content.get_state_mut(rich_text_id) {
             rte.layout.font_size = font_size;
             rte.update_font_size();
 
@@ -95,12 +115,7 @@ impl SugarState {
         operation: u8,
         advance_brush: &mut RichTextBrush,
     ) {
-        if let Some(rte) = self
-            .compositors
-            .advanced
-            .content
-            .get_state_mut(rich_text_id)
-        {
+        if let Some(rte) = self.content.get_state_mut(rich_text_id) {
             let should_update = match operation {
                 0 => rte.reset_font_size(),
                 2 => rte.increase_font_size(),
@@ -118,12 +133,16 @@ impl SugarState {
         self.process_rich_text_repaint(advance_brush);
     }
 
+    #[inline]
+    pub fn set_rich_text_line_height(&mut self, rich_text_id: &usize, line_height: f32) {
+        if let Some(rte) = self.content.get_state_mut(rich_text_id) {
+            rte.layout.line_height = line_height;
+        }
+    }
+
     fn process_rich_text_repaint(&mut self, advance_brush: &mut RichTextBrush) {
         for rich_text in &self.rich_text_repaint {
-            self.compositors
-                .advanced
-                .content
-                .update_dimensions(rich_text, advance_brush);
+            self.content.update_dimensions(rich_text, advance_brush);
         }
 
         self.rich_text_repaint.clear();
@@ -131,8 +150,8 @@ impl SugarState {
 
     #[inline]
     pub fn set_fonts(&mut self, fonts: &FontLibrary, advance_brush: &mut RichTextBrush) {
-        self.compositors.advanced.set_fonts(fonts);
-        for (id, state) in &mut self.compositors.advanced.content.states {
+        self.content.set_font_library(fonts);
+        for (id, state) in &mut self.content.states {
             state.layout.dimensions.height = 0.0;
             state.layout.dimensions.width = 0.0;
             self.rich_text_repaint.insert(*id);
@@ -143,7 +162,8 @@ impl SugarState {
 
     #[inline]
     pub fn set_font_features(&mut self, font_features: &Option<Vec<String>>) {
-        self.compositors.advanced.set_font_features(font_features);
+        let found_font_features = SugarState::found_font_features(font_features);
+        self.content.set_font_features(found_font_features);
     }
 
     #[inline]
@@ -166,61 +186,58 @@ impl SugarState {
     }
 
     #[inline]
-    pub fn reset_compositors(&mut self) {
-        self.compositors.elementary.clean();
+    pub fn reset(&mut self) {
+        self.quads.clear();
+        for rte_id in &self.rich_text_to_be_removed {
+            self.content.remove_state(rte_id);
+        }
+
+        self.content.mark_states_clean();
+        self.rich_text_to_be_removed.clear();
     }
 
     #[inline]
     pub fn clear_rich_text(&mut self, id: &usize) {
-        self.compositors.advanced.clear_rich_text(id);
+        self.content.clear_state(id);
     }
 
     #[inline]
     pub fn create_rich_text(&mut self) -> usize {
-        self.compositors
-            .advanced
-            .create_rich_text(&RichTextLayout::from_default_layout(&self.style))
+        self.content
+            .create_state(&RichTextLayout::from_default_layout(&self.style))
+    }
+
+    #[inline]
+    pub fn create_temp_rich_text(&mut self) -> usize {
+        let id = self
+            .content
+            .create_state(&RichTextLayout::from_default_layout(&self.style));
+        self.rich_text_to_be_removed.push(id);
+        id
     }
 
     pub fn content(&mut self) -> &mut Content {
-        &mut self.compositors.advanced.content
+        &mut self.content
     }
 
     #[inline]
     pub fn compute_updates(
         &mut self,
         advance_brush: &mut RichTextBrush,
-        elementary_brush: &mut text::GlyphBrush<()>,
-        rect_brush: &mut RectBrush,
         quad_brush: &mut QuadBrush,
         context: &mut super::Context,
         graphics: &mut Graphics,
     ) {
         advance_brush.prepare(context, self, graphics);
-        rect_brush.resize(context);
         quad_brush.resize(context);
 
         // Elementary renderer is used for everything else in sugarloaf
         // like objects rendering (created by .text() or .append_rects())
-        // ...
-        // If current tree has objects and compositor has empty objects
         // It means that's either the first render or objects were erased on compute_diff() step
         for object in &self.objects {
             match object {
-                Object::Text(text) => {
-                    elementary_brush.queue(
-                        &self.compositors.elementary.create_section_from_text(
-                            text,
-                            context,
-                            &self.style,
-                        ),
-                    );
-                }
-                Object::Rect(rect) => {
-                    self.compositors.elementary.rects.push(*rect);
-                }
                 Object::Quad(composed_quad) => {
-                    self.compositors.elementary.quads.push(*composed_quad);
+                    self.quads.push(*composed_quad);
                 }
                 Object::RichText(_rich_text) => {
                     // self.rich_texts.push(*rich_text);
@@ -235,11 +252,8 @@ impl SugarState {
         id: &usize,
         advance_brush: &mut RichTextBrush,
     ) -> SugarDimensions {
-        self.compositors
-            .advanced
-            .content
-            .update_dimensions(id, advance_brush);
-        if let Some(rte) = self.compositors.advanced.content.get_state(id) {
+        self.content.update_dimensions(id, advance_brush);
+        if let Some(rte) = self.content.get_state(id) {
             return rte.layout.dimensions;
         }
 
@@ -250,8 +264,7 @@ impl SugarState {
     pub fn compute_dimensions(&mut self, advance_brush: &mut RichTextBrush) {
         // If sugar dimensions are empty then need to find it
         for rich_text in &self.rich_texts {
-            if let Some(rte) = self.compositors.advanced.content.get_state(&rich_text.id)
-            {
+            if let Some(rte) = self.content.get_state(&rich_text.id) {
                 if rte.layout.dimensions.width == 0.0
                     || rte.layout.dimensions.height == 0.0
                 {
@@ -266,12 +279,14 @@ impl SugarState {
             return;
         }
         for rich_text in &self.rich_text_repaint {
-            self.compositors
-                .advanced
-                .content
-                .update_dimensions(rich_text, advance_brush);
+            self.content.update_dimensions(rich_text, advance_brush);
         }
 
         self.rich_text_repaint.clear();
     }
+
+    #[inline]
+    pub fn set_visual_bell_overlay(&mut self, overlay: Option<Quad>) {
+        self.visual_bell_overlay = overlay;
+    }
 }
